unit DataStructures;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, MSXML_TLB;

type
  TFMMain = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    Label1: TLabel;
    Label3: TLabel;
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    Label5: TLabel;
    EditArrayStringToSet: TEdit;
    ButtonArraySet: TButton;
    ListBoxItemsInArray: TListBox;
    MemoXMLDataForArray: TMemo;
    Label4: TLabel;
    Label6: TLabel;
    Label2: TLabel;
    Panel2: TPanel;
    Label7: TLabel;
    GroupBox2: TGroupBox;
    Label8: TLabel;
    EditListStringToAdd: TEdit;
    ButtonListAdd: TButton;
    ListBoxItemsInList: TListBox;
    Label9: TLabel;
    MemoXMLDataForList: TMemo;
    Panel3: TPanel;
    Label10: TLabel;
    GroupBox3: TGroupBox;
    Label11: TLabel;
    EditTreeStringToAdd: TEdit;
    ButtonTreeAddSibling: TButton;
    Label12: TLabel;
    MemoXMLDataForTree: TMemo;
    TreeViewItems: TTreeView;
    ButtonTreeAddChild: TButton;
    procedure ListBoxItemsInArrayKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ButtonArraySetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonListAddClick(Sender: TObject);
    procedure ListBoxItemsInListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ButtonTreeAddSiblingClick(Sender: TObject);
    procedure TreeViewItemsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ButtonTreeAddChildClick(Sender: TObject);

  private
    XMLDOMArray: IXMLDOMDocument;
    XMLDOMList:  IXMLDOMDocument;
    XMLDOMTree:  IXMLDOMDocument;

    procedure DeleteListBoxItem(TheListBox: TListBox);
    
    procedure DeleteArrayXMLItem(TheIndex: Integer);
    procedure SetArrayXMLItem(TheIndex: Integer; TheItem: String);
    procedure DisplayArrayXML;
    
    procedure DeleteListXMLItem(TheIndex: Integer);
    procedure AddListXMLItem(TheIndex: Integer; TheItem: String);
    procedure DisplayListXML;

    procedure DeleteTreeXMLItem(TheTreeViewNode: TTreeNode);
    procedure AddTreeXMLItem(TheParentNode: TTreeNode; TheText: String);
    procedure DisplayTreeXML;
    function FindNodeWithAbsoluteIndex(TheAbsoluteIndex: Integer): IXMLDOMNode;
  end;

var
  FMMain: TFMMain;

implementation

{$R *.DFM}

procedure TFMMain.FormCreate(Sender: TObject);
var
  Node: IXMLDOMNode;
begin
  XMLDOMArray := CoDOMDocument.Create;
  XMLDOMArray.async := False;
  XMLDOMArray.validateOnParse := True;
  Node := XMLDOMArray.createNode(NODE_ELEMENT, 'Array', '');
  XMLDOMArray.appendChild(Node);
  DisplayArrayXML;

  XMLDOMList := CoDOMDocument.Create;
  XMLDOMList.async := False;
  XMLDOMList.validateOnParse := True;
  Node := XMLDOMList.createNode(NODE_ELEMENT, 'List', '');
  XMLDOMList.appendChild(Node);
  DisplayListXML;

  XMLDOMTree := CoDOMDocument.Create;
  XMLDOMTree.async := False;
  XMLDOMTree.validateOnParse := True;
  Node := XMLDOMTree.createNode(NODE_ELEMENT, 'Tree', '');
  XMLDOMTree.appendChild(Node);
  DisplayTreeXML;
end;

procedure TFMMain.DeleteListBoxItem(TheListBox: TListBox);
var
  I: Integer;
begin
  I := TheListBox.ItemIndex;
  if I < 0 then Exit;
  TheListBox.Items.Delete(TheListBox.ItemIndex);
  if I < TheListBox.Items.Count then
    TheListBox.ItemIndex := I
  else
    TheListBox.ItemIndex := TheListBox.Items.Count-1;
end;

{******************************************************************************}
{*                              Array methods                                 *}
{******************************************************************************}

procedure TFMMain.ButtonArraySetClick(Sender: TObject);
var
  I: Integer;
begin
  I := ListBoxItemsInArray.ItemIndex;
  ListBoxItemsInArray.Items.Delete(I);
  ListBoxItemsInArray.Items.Insert(I, EditArrayStringToSet.Text);
  SetArrayXMLItem(I, EditArrayStringToSet.Text);
end;

procedure TFMMain.SetArrayXMLItem(TheIndex: Integer; TheItem: String);
var
  ArrayNode, Node, NewNode: IXMLDOMNode;
begin
  ArrayNode := XMLDOMArray.documentElement;
  Node := ArrayNode.childNodes [TheIndex];
  if Node = nil then
  begin
    NewNode := XMLDOMArray.createElement('Item');
    NewNode.text := TheItem;
    ArrayNode.appendChild(NewNode);
  end else
    Node.text := TheItem;

  DisplayArrayXML;
end;

procedure TFMMain.DeleteArrayXMLItem(TheIndex: Integer);
var
  ArrayNode, Node: IXMLDOMNode;
begin
  ArrayNode := XMLDOMArray.documentElement;
  Node := ArrayNode.childNodes [TheIndex];
  if Node = nil then Exit;
  Node.text := '';
  DisplayArrayXML;
end;

procedure TFMMain.DisplayArrayXML;
begin
  MemoXMLDataForArray.Lines.Clear;
  MemoXMLDataForArray.Lines.Add(XMLDOMArray.xml);
end;

procedure TFMMain.ListBoxItemsInArrayKeyDown(Sender: TObject;
                                             var Key: Word;
                                             Shift: TShiftState);
var
  I: Integer;
begin
  if Key <> VK_DELETE then Exit;
  DeleteArrayXMLItem(ListBoxItemsInArray.ItemIndex);
  I := ListBoxItemsInArray.ItemIndex;
  if I < 0 then Exit;
  ListBoxItemsInArray.Items [I] := '';
end;

{******************************************************************************}
{*                               List methods                                 *}
{******************************************************************************}

procedure TFMMain.ButtonListAddClick(Sender: TObject);
var
  I: Integer;
begin
  I := ListBoxItemsInList.ItemIndex;
  ListBoxItemsInList.Items.Insert(I, EditListStringToAdd.Text);
  AddListXMLItem(I, EditListStringToAdd.Text);
end;

procedure TFMMain.AddListXMLItem(TheIndex: Integer; TheItem: String);
var
  ListNode, Node, NewNode: IXMLDOMNode;
begin
  ListNode := XMLDOMList.documentElement;
  NewNode := XMLDOMList.createElement('Item');
  NewNode.text := TheItem;
  Node := ListNode.childNodes [TheIndex];
  if Node = nil then
    ListNode.appendChild(NewNode)
  else
    ListNode.insertBefore(NewNode, Node);

  DisplayListXML;
end;

procedure TFMMain.DeleteListXMLItem(TheIndex: Integer);
var
  ListNode, Node: IXMLDOMNode;
begin
  ListNode := XMLDOMList.documentElement;
  Node := ListNode.childNodes [TheIndex];
  if Node = nil then Exit;
  ListNode.removeChild(Node);
  DisplayListXML;
end;

procedure TFMMain.DisplayListXML;
begin
  MemoXMLDataForList.Lines.Clear;
  MemoXMLDataForList.Lines.Add(XMLDOMList.xml);
end;

procedure TFMMain.ListBoxItemsInListKeyDown(Sender: TObject;
                                            var Key: Word;
                                            Shift: TShiftState);
begin
  if Key <> VK_DELETE then Exit;
  DeleteListXMLItem(ListBoxItemsInList.ItemIndex);
  DeleteListBoxItem(ListBoxItemsInList);
end;

{******************************************************************************}
{*                               Tree methods                                 *}
{******************************************************************************}

procedure TFMMain.ButtonTreeAddSiblingClick(Sender: TObject);
var
  Node, ParentNode: TTreeNode;
begin
  Node := TreeViewItems.Selected;
  if Node = nil then
    ParentNode := nil
  else
    ParentNode := Node.Parent;

  if Node = nil then
    TreeViewItems.Items.AddChild(nil, EditTreeStringToAdd.Text)
  else
    TreeViewItems.Items.AddChild(Node.Parent, EditTreeStringToAdd.Text);
  TreeViewItems.FullExpand;
  AddTreeXMLItem(ParentNode, EditTreeStringToAdd.Text);
end;

procedure TFMMain.ButtonTreeAddChildClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  Node := TreeViewItems.Selected;
  TreeViewItems.Items.AddChild(Node, EditTreeStringToAdd.Text);
  TreeViewItems.FullExpand;
  AddTreeXMLItem(Node, EditTreeStringToAdd.Text);
end;

procedure TFMMain.AddTreeXMLItem(TheParentNode: TTreeNode; TheText: String);
var
  TreeNode, Node, NewNode: IXMLDOMNode;
  NodeText: String;
  Level, ParentIndex: Integer;
begin
  if TheParentNode = nil then
  begin
    Level := 0;
    ParentIndex := 0;
  end else
  begin
    Level := TheParentNode.Level + 1;
    ParentIndex := TheParentNode.AbsoluteIndex;
  end;

  Inc(Level);  // make level numbers start at 1
  TreeNode := XMLDOMTree.documentElement;
  NodeText := Format('Level%d', [Level] );
  NewNode := XMLDOMTree.createElement(NodeText);
  NewNode.text := TheText;

  Node := FindNodeWithAbsoluteIndex(ParentIndex);

  if TheParentNode = nil then
    TreeNode.appendChild(NewNode)
  else
    Node.appendChild(NewNode);

  DisplayTreeXML;
end;

// recursively descend the DOM tree until the i-th element is found
function TFMMain.FindNodeWithAbsoluteIndex(TheAbsoluteIndex: Integer): IXMLDOMNode;
var
  XMLDOMNodeList: IXMLDOMNodeList;
begin
  XMLDOMNodeList := XMLDOMTree.documentElement.getElementsByTagName('*');
  Result := XMLDOMNodeList.item [TheAbsoluteIndex];
end;

procedure TFMMain.DeleteTreeXMLItem(TheTreeViewNode: TTreeNode);
var
  Index: Integer;
  TreeNode, Node: IXMLDOMNode;
begin
  if TheTreeViewNode = nil then
    Index := 0
  else
    Index := TheTreeViewNode.AbsoluteIndex;

  TreeNode := XMLDOMTree.documentElement;
  Node := FindNodeWithAbsoluteIndex(Index);
  if Node = nil then Exit;

  if Index = 0 then
    TreeNode.removeChild(Node)
  else
    Node.parentNode.removeChild(Node);
  DisplayTreeXML;
end;

procedure TFMMain.DisplayTreeXML;
begin
  MemoXMLDataForTree.Lines.Clear;
  MemoXMLDataForTree.Lines.Add(XMLDOMTree.xml);
end;

procedure TFMMain.TreeViewItemsKeyDown(Sender: TObject;
                                       var Key: Word;
                                       Shift: TShiftState);
var
  Node: TTreeNode;
begin
  if Key <> VK_DELETE then Exit;

  Node := TreeViewItems.Selected;
  if Node = nil then Exit;

  DeleteTreeXMLItem(Node);
  TreeViewItems.Selected := Node.Parent;
  TreeViewItems.Items.Delete(Node);
end;

end.
