unit DisplayXML;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MSXML2_TLB;

  procedure HTMLforNode(TheHTMLText: TStrings; TheXmlNode: IXMLDOMNode);
  procedure HTMLforNodeList(TheHTMLText: TStrings; TheXmlNodeList: IXMLDOMNodeList);
  procedure DumpTree(TheNode: IXMLDOMNode; TheHTMLText: TStrings);

implementation

// this code is based on the Microsoft JavaScript example at
// http://msdn.microsoft.com/downloads/samples/internet/xml/tree_viewer/default.asp

procedure ProcessAttributes(TheNode: IXMLDOMNode; TheHTMLText: TStrings);
var
  I, L: Integer;
  A: IXMLDOMNode;
begin
  if TheNode.attributes.length = 0 then Exit;
  L := TheNode.attributes.length;
  for I := 0 to L-1 do
  begin
    A := TheNode.attributes.item [I];
    TheHTMLText.Add('<span class=attr> ' + a.nodeName + '="' + a.text + '"</span>');
  end;
end;

function IsMixedNode(TheNode: IXMLDOMNode): Boolean;
var
  I: Integer;
  Child: IXMLDOMNode;
begin
  Result := True;
  for I := 0 to TheNode.childNodes.length-1 do
  begin
    Child := TheNode.childNodes.item [I];
    if (child.nodeTypeString = 'text') or
       (child.nodeTypeString = 'cdata_section') or
       (child.nodeTypeString = 'entity_reference') then
      Exit;
  end;
  Result := False;
end;

procedure ProcessChildren(TheNode: IXMLDOMNode; TheHTMLText: TStrings);
var
  I, Num: Integer;
begin
  if TheNode.childNodes <> nil then
    Num := TheNode.childNodes.length
  else
    Num := 0;

  // close the element tag (if empty, use shorthand)

  if Num = 0 then       // tag is empty
    TheHTMLText.Add('<span class=tag>/&gt;</span>')
  else
    TheHTMLText.Add('<span class=tag>&gt;</span>');

  // process the children of the element if it has any
  if Num > 0 then    // tag has children
  begin
    if IsMixedNode(TheNode) then
      TheHTMLText.Add(TheNode.text)
    else
      for I := 0 to Num-1 do
        DumpTree(TheNode.childNodes.item [I], TheHTMLText);
    TheHTMLText.Add('<span class=tag>&lt;/' + TheNode.nodeName + '&gt;</span>');
  end;
end;

// Format the XML into HTML
procedure DumpTree(TheNode: IXMLDOMNode; TheHTMLText: TStrings);
begin
  if TheNode = nil then
  begin
    TheHTMLText.Add('<DL class=xml><DD>');
    TheHTMLText.Add('</DD></DL>');
    Exit;
  end;

  if TheNode.nodeTypeString = 'comment' then
  begin
    TheHTMLText.Add('<DL class=xml><DD>');
    TheHTMLText.Add('<span class=comment>&lt;!--' + TheNode.text + '--&gt;</span>' + '</DD></DL>');
    TheHTMLText.Add('</DD></DL>');
    TheHTMLText.Add('</DD></DL>');
    Exit;
  end;

  TheHTMLText.Add('<DL class=xml><DD>');
  TheHTMLText.Add('<span class=tag>&lt;' + TheNode.nodeName + '</span>');
  ProcessAttributes(TheNode, TheHTMLText);
  ProcessChildren(TheNode, TheHTMLText);
  TheHTMLText.Add('</DD></DL>');
end;

procedure AddStyleSheet(TheHTMLText: TStrings);
begin
  TheHTMLText.Add('<STYLE TYPE="text/css">');
  TheHTMLText.Add('.xml {font-weight: bold}');
  TheHTMLText.Add('.tag {color: green}');
  TheHTMLText.Add('</STYLE>');
end;

procedure HTMLforNode(TheHTMLText: TStrings; TheXmlNode: IXMLDOMNode);
begin
  TheHTMLText.Add('<HTML>');
  AddStyleSheet(TheHTMLText);
  DumpTree(TheXmlNode, TheHTMLText);
  TheHTMLText.Add('</HTML>');
end;

procedure HTMLforNodeList(TheHTMLText: TStrings; TheXmlNodeList: IXMLDOMNodeList);
var
  I: Integer;
begin
  TheHTMLText.Add('<HTML>');
  AddStyleSheet(TheHTMLText);
  for I := 0 to TheXmlNodeList.length-1 do
    DumpTree(TheXmlNodeList.item [I], TheHTMLText);
  TheHTMLText.Add('</HTML>');
end;

end.
